#ifndef GC_GC_H
#define GC_GC_H

#include <vector>
#include "defs/singleton.h"
#include "gc/link.h"
#include "gc/pointer.h"

namespace gc {

class Object;

//what if, instead of GlobalPointer's with null owner's
//we just make GarbageCollectSystem a gc::Object of its own?
//...then it'd get linked upon construction to itself though ...
class GarbageCollectSystem {
	friend class Object;
	friend class PointerBase;
protected:
	Link ptrs;	//all ptrs not members of objects (and without owners) go here
	Link objs;
public:
	void collect();
	void markPtrList(Link *ptrs, std::vector<Object*> *marked);

	int getNumPtrs() const { return ptrs.size(); }
	int getNumObjs() const { return objs.size(); }
public:
	template<typename T>
	GlobalPointer<T> createPtr(T *_ptr = NULL) {
		GlobalPointer<T> p(_ptr);
		p.attach(&ptrs);
		return p;
	}
};

//hmm, global static?  compile order?
//how about a singleton!

extern Singleton<GarbageCollectSystem> sys;

};

#endif	//GC_GC_H
